@@ -1,12 +1,15 @@
 """
 	Redid CLI tools entry point.
 """
-import sys, os, yaml, argparse, json, mimetypes, urllib
+import sys, os, yaml, argparse, json, mimetypes, urllib, fnmatch
+import dateutil.parser
 
+from tabulate import tabulate 
 from common import log
 import common
 
 verbose = False
+show_curl = False
 config = None
 	
 def cmd_setup( args ):
@@ -44,10 +47,14 @@ def cmd_get_profile( args ):
 	
 def cmd_put_profile( args ):
 	profile = args.profile
-	if profile == None:
-		profile = os.path.splitext( os.path.basename( args.load ) )[0]
 		
-	data = load_data( args.load )
+	data, name = load_data( args.file )
+	if profile == None:
+		if name == None:
+			log( "A name must be specified for the profile if not using a file" )
+			return False
+		profile = name
+			
 	r = api_request( 'PUT', 'profile/' + profile, json_data = data )
 	return handle_object_response( args, r )
 	
@@ -82,7 +89,7 @@ def cmd_list_profiles( args ):
 		handle_data( args, names )
 	return True
 	
-def cmd_list_resources( args ):
+def get_resources( args ):
 	query = {}
 	if args.sha256 != None:
 		query['sha256'] = args.sha256
@@ -92,17 +99,23 @@ def cmd_list_resources( args ):
 	resources = []
 	r = api_request( 'GET', 'resource-list', query_string = query )
 	if r == None:
-		return False
+		return None
 	resources = resources + r['data']
-
+	
 	while 'next' in r:
 		query['next'] = r['next']
 		r = api_request( 'GET', 'resource-list', query_string = query )
 		if r == None:
-			return False
+			return None
 		resources = resources + r['data']
-		
-	if args.full:
+	return resources
+	
+def cmd_list_resources( args ):
+	resources = get_resources( args )
+	if resources == None:
+		return False
+	
+	elif args.full:
 		mapped = { (r['name'] if 'name' in r else r['id']): r for r in resources  }
 		handle_data( args, mapped )
 	else:
@@ -112,6 +125,47 @@ def cmd_list_resources( args ):
 	return True
 	
 	
+def cmd_ls_resources( args ):
+	# if using a filter we can speed up if there is a name-prefix (exclude _ since it won't match any names
+	# on the server)
+	if args.filter != None and args.name_prefix == None and not args.filter.startswith('_'):
+		p = args.filter.split( '*', 1 )[0]
+		p = p.split( '?', 1 )[0]
+		if len(p) > 0:
+			args.name_prefix = p
+		
+	resources = get_resources( args )
+	if resources == None:
+		return False
+	
+	def title( r ):
+		return r['name'] if 'name' in r else r['id']
+		
+	def clean_date( s ):
+		d = dateutil.parser.parse( s )
+		return d.strftime( '%Y-%m-%d %H:%M' )
+		
+	def filtered( r ):
+		if args.filter == None:
+			return True
+		return fnmatch.fnmatchcase( title(r), args.filter )
+		
+	def sort( x, y ):
+		if args.reverse:
+			x,y = y,x
+			
+		if args.sort == 'name':
+			return cmp( title(x), title(y) )
+		return cmp( x[args.sort], y[args.sort] )
+		
+	table = [ (r, r['resource_type'], r['type'], r['bytesize'], clean_date(r['modified']), title(r))
+		for r in resources if filtered(r) ]
+	s = sorted( table, cmp = lambda x,y: sort(x[0],y[0]) )
+	print( tabulate( [ v[1:] for v in s], tablefmt = 'plain' ) )
+	
+	return True
+	
+	
 def cmd_get_resource_record( args ):
 	r = api_request( 'GET', 'resource-record/' + args.path )
 	return handle_object_response( args, r )
@@ -152,8 +206,7 @@ def cmd_upload_resource( args ):
 		method = 'POST'
 		url = 'resource'
 	
-	with open( args.local_file, 'rb' ) as fp:
-		r = api_request( method, url, query_string = query, data = fp.read(), headers = headers )
+	r = api_request( method, url, query_string = query, data_file = args.local_file, headers = headers )
 	return handle_object_response( args, r )
 	
 	
@@ -196,6 +249,8 @@ def main():
 	opts = argparse.ArgumentParser()
 	common.add_config_args( opts )
 	opts.add_argument( "--verbose", help = "show verbose output", action='store_const', const = True )
+	opts.add_argument( "--show-curl", help = "show the calls made in curl CLI syntax", 
+		action='store_const', const = True )
 	opts.add_argument( "--save", help = "save the command result to this file instead of displaying",
 		nargs = "?" )
 	sub = opts.add_subparsers( help = 'command help', title = "Commands", dest = 'command' )
@@ -220,7 +275,7 @@ def main():
 	o.add_argument( "profile" )
 		
 	o = add_local( 'put-profile', 'create/update a profile' )
-	o.add_argument( 'load', help = 'filename of the profile to upload' )
+	o.add_argument( 'file', help = 'filename of the profile to upload' )
 	o.add_argument( 'profile', help = 'name of profile, if not specified basename of file is used',
 		nargs = '?' )
 		
@@ -236,11 +291,20 @@ def main():
 	
 	
 	# Resource commands
+	def add_common_resource(o):
+		o.add_argument( '--sha256', help = 'matching the given hash (hex encoded)' )
+		o.add_argument( '--name-prefix', help = 'starting with this name prefix' )
+		
 	o = add_local( 'list-resources', 'get a listing of resources' )
-	o.add_argument( '--sha256', help = 'matching the given hash (hex encoded)' )
-	o.add_argument( '--name-prefix', help = 'starting with this name prefix' )
+	add_common_resource(o)
 	o.add_argument( '--full', help = 'output the resource record, not just the name', 
 		action = 'store_const', const = True )
+		
+	o = add_local( 'ls-resources', 'human friendly resource listing' )
+	add_common_resource(o)
+	o.add_argument( '--sort', help = 'sort on this resource field', default = 'name' )
+	o.add_argument( '--reverse', help = 'reversed sorting', action = 'store_const', const = True )
+	o.add_argument( 'filter', help = 'wildcard filter to limit output', nargs = '?' )
 	
 	o = add_local( 'get-resource-record', 'get the resource record', with_path = True )
 	
@@ -274,8 +338,9 @@ def main():
 	
 	# args parsing
 	args = opts.parse_args()
-	global verbose
+	global verbose, show_curl
 	verbose = args.verbose
+	show_curl = args.show_curl
 	
 	if args.command == 'setup':
 		args.func( args )
@@ -290,7 +355,7 @@ def main():
 
 		
 def api_request( *args, **kwargs ):
-	return common.api_request( *args, config = config, verbose = verbose, **kwargs )
+	return common.api_request( *args, config = config, verbose = verbose, show_curl = show_curl, **kwargs )
 
 	
 def handle_object_response( args, data ):
@@ -331,9 +396,20 @@ def save_data( data, filename, binary = False ):
 	
 	
 def load_data( filename ):
+	"""
+		Loads data from a file and returns the basename of that file
+	"""
+	# Allows data to be put on the command-line
+	if filename.startswith( 'json:' ):
+		return json.loads( filename[5:] ), None
+	if filename.startswith( 'yaml:' ):
+		return yaml.load( filename[5:] ), None
+
+	base = os.path.basename( filename )
+	profile = os.path.splitext( base )[0]
+		
 	if filename.endswith( '.yaml' ):
-		return yaml.load( file( filename, 'r' ) )
-	elif filename.endswith( '.json' ):
-		return json.load( file( filename, 'r' ) )
-	else:
-		raise Exception( "unknown-file-type: " + filename )
+		return yaml.load( file( filename, 'r' ) ), base
+	if filename.endswith( '.json' ):
+		return json.load( file( filename, 'r' ) ), base
+	raise Exception( "unknown-file-type: " + filename )
